/* --------------------------------------------------------------  */
/* (C)Copyright 2001,2008,                                         */
/* International Business Machines Corporation,                    */
/* Sony Computer Entertainment, Incorporated,                      */
/* Toshiba Corporation,                                            */
/*                                                                 */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#ifndef _DIVI4_H_
#define _DIVI4_H_	1

#include <altivec.h>
#include "simdmath.h"

/*
 * FUNCTION
 * 	vector divi4_t _divi4(vector signed int dividend, 
 *				      vector signed int divisor)
 * 
 * DESCRIPTION
 *	The _divi4 subroutine computes 4 simultaneous signed integer
 *	quotients and remainders by dividing each component of the vector 
 *	dividend by the correspoding component of the vector divisor. If
 *	the divisor is 0, then a quotient of 0 is produced.
 *
 *	Zero is also produced when 0x80000000 is divided by -1. 
 *
 */
static __inline divi4_t _divi4(vector signed int dividend, vector signed int divisor)
{
  /* On the PU, it is likely more effective to perform each element using 
   * the PPC scalar divide.
   */
  divi4_t result;
  union {
    vector signed int v;
    signed int i[4];
  } u1, u2, u3, u4;

  u1.v = dividend;
  u2.v = divisor;

  u3.i[0] = u1.i[0] / u2.i[0];
  u3.i[1] = u1.i[1] / u2.i[1];
  u3.i[2] = u1.i[2] / u2.i[2];
  u3.i[3] = u1.i[3] / u2.i[3];

  u4.i[0] = u1.i[0] - u3.i[0] * u2.i[0];
  u4.i[1] = u1.i[1] - u3.i[1] * u2.i[1];
  u4.i[2] = u1.i[2] - u3.i[2] * u2.i[2];
  u4.i[3] = u1.i[3] - u3.i[3] * u2.i[3];

  result.quot = u3.v;
  result.rem  = u4.v;
  return (result);
}

#endif /* _DIVI4_H_ */
