/* --------------------------------------------------------------  */
/* (C)Copyright 2001,2008,                                         */
/* International Business Machines Corporation,                    */
/* Sony Computer Entertainment, Incorporated,                      */
/* Toshiba Corporation,                                            */
/*                                                                 */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#ifndef _FMAXF4_H_
#define _FMAXF4_H_	1

#include <altivec.h>
#include <vec_types.h>

/*
 * FUNCTION
 *	vector float _fmaxf4(vector float x, vector float y)
 *
 * DESCRIPTION
 *	The _fmaxf4 function returns the maximum numeric value of each 
 *      element of x and y.  
 *
 *      Note, this function is more complex and IEEE-correct than
 *      simply calling vec_max(x,y).  This function coerces denormals to 
 *      zero, and handles NaNs and Infinities on inputs more correctly 
 *      than vec_max(x,y).  This function does not return qNaNs however, 
 *      which a fully compliant version would.
 *
 */
static __inline vector float _fmaxf4(vector float x, vector float y)
{
  vec_uint4 sign_mask = (vec_uint4) { 0x80000000, 0x80000000, 0x80000000, 0x80000000 };
  vec_uint4 nan_threshold = (vec_uint4) { 0x7F800000, 0x7F800000, 0x7F800000, 0x7F800000 };

  vec_uint4 abs_y = vec_andc((vec_uint4)y,sign_mask);

  vec_float4 result = vec_sel(y,x,vec_or(vec_cmpgt(abs_y,nan_threshold),vec_cmpge(x,y)));
  
  return vec_add(result,(vec_float4)sign_mask);
}

#endif /* _FMAXF4_H_ */

