/* --------------------------------------------------------------  */
/* (C)Copyright 2006,2008,                                         */
/* International Business Machines Corporation                     */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#ifndef _ISLESSGREATERF4_H_
#define _ISLESSGREATERF4_H_	1

#include <altivec.h>
#include <vec_types.h>

#include "isunorderedf4.h"

/*
 * FUNCTION
 *	vector unsigned int _islessgreaterf4(vector float x, vector float y)
 *
 * DESCRIPTION
 *      The _islessgreaterf4 function returns a vector in which each element 
 *      indicates if the corresponding element of x is less than or greater 
 *      than the corresponding element of y. 
 *
 * RETURNS
 *      UINT_MAX (0xFFFFFFFF)  if the element of x is less than or greater
 *                             than the element of y.
 *      0        (0x00000000)  otherwise
 *
 */
static __inline vector unsigned int _islessgreaterf4(vector float x, vector float y)
{

  // if:
  //  1)  not unorderer
  //  2)  bit equal
  //  3)  some sort of zero

  vec_uint4 zeros = (vec_uint4)vec_cmpeq(vec_sl(vec_or((vec_uint4)x,(vec_uint4)y),vec_splat_u32(1)),vec_splat_u32(0));

  vec_uint4 equal = (vec_uint4)vec_cmpeq((vec_uint4)x,(vec_uint4)y);

  vec_uint4 unordered = _isunorderedf4(x,y);

  return vec_sel(vec_nor(equal, zeros),vec_splat_u32(0),unordered);

  
}

#endif /* _ISLESSGREATERF4_H_ */
