/* --------------------------------------------------------------  */
/* (C)Copyright 2007,2008,                                         */
/* International Business Machines Corporation                     */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#ifndef _ISNORMALF4_H_
#define _ISNORMALF4_H_	1

#include <altivec.h>

/*
 * FUNCTION
 *      vector unsigned int _isnormalf4(vector float x)
 *
 * DESCRIPTION
 *      The _isnormalf4 function returns a vector in which each element 
 *      indicates if the corresponding element of x is normal (not subnormal, 
 *      +infinite, -infinite, NaN).
 *
 * RETURNS
 *      UINT_MAX (0xFFFFFFFF)  if the element of x is normal
 *      0        (0x00000000)  otherwise
 *
 */
static __inline vector unsigned int _isnormalf4(vector float x)
{
   vector unsigned int low  = (vector unsigned int) {0x007FFFFF,0x007FFFFF,0x007FFFFF,0x007FFFFF};
   vector unsigned int inf  = (vector unsigned int) {0x7F800000,0x7F800000,0x7F800000,0x7F800000};
   vector unsigned int xabs = (vector unsigned int) vec_abs(x);
   vector unsigned int zero = vec_xor(low,low);


   return (vector unsigned int)(vec_or(vec_cmpeq(xabs,zero),             /* If the abs val of x is zero, or */
				       (vec_and(vec_cmpgt(xabs,low),      /* (larger than the biggest denorm AND */
						vec_cmplt(xabs,inf)))));  /* smaller than infinity), return ones */
}

#endif /* _ISNORMALF4_H_ */

