/* --------------------------------------------------------------  */
/* (C)Copyright 2006,2008,                                         */
/* International Business Machines Corporation,                    */
/* Sony Computer Entertainment, Incorporated,                      */
/* Toshiba Corporation,                                            */
/*                                                                 */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#ifndef _ROUNDF4_H_
#define _ROUNDF4_H_	1

#include <altivec.h>

/* 
 * FUNCTION
 *      vector float _roundf4(vector float x)
 *
 * DESCRIPTION
 *      The _roundf4 function rounds the elments of the input vector to the 
 *      nearest integer, rounding halfway cases away from zero.
 *
 */
static __inline vector float _roundf4(vector float x)
{
  vector unsigned int sign_mask = (vector unsigned int) { 0x80000000, 0x80000000, 0x80000000, 0x80000000 };

  vector unsigned int emax = (vector unsigned int) { 0x4B000000, 0x4B000000, 0x4B000000, 0x4B000000};

  //  Bias by one ULP less than 0.5.  This corrects for adding error
  vector unsigned int almosthalf = (vector unsigned int) { 0x3EFFFFFF, 0x3EFFFFFF, 0x3EFFFFFF, 0x3EFFFFFF };

  //  Convert ~0.5 to ~-0.5 if the sign bit is set
  vector unsigned int addin = vec_or(vec_and((vector unsigned int)x,sign_mask),almosthalf);

  //  Set the correction to ~+- 0.5 or 0, depending if the exponent is greater than 23
  addin = vec_and(addin,vec_cmplt(vec_andc((vector unsigned int)x,sign_mask),emax));

  //  Add the correction and truncate
  return vec_trunc(vec_add(x,(vector float)addin));

  
}

#endif /* _ROUNDF4_H_ */
