/* --------------------------------------------------------------  */
/* (C)Copyright 2001,2008,                                         */
/* International Business Machines Corporation,                    */
/* Sony Computer Entertainment, Incorporated,                      */
/* Toshiba Corporation,                                            */
/*                                                                 */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#ifndef _RSQRTF4_H_
#define _RSQRTF4_H_	1

#include <altivec.h>

/*
 * FUNCTION
 *	vector float _rsqrtf4(vector float value)
 *
 * DESCRIPTION
 *	The _rsqrtf4 function computes 1 over the square root of the vector 
 *      input "value" and returns the result. Computation is performed using
 *      the floating-point reciprocal square root estimate and interpolate 
 *      instructions to produce a 12 bit accurate estimate. One iteration of a
 *      Newton-Raphson is performed to improve accuracy to single precision 
 *      floating point. 
 *
 *	The Newton-Raphson iteration is of the form:
 *		X[i+1] = 0.5 * X[i] * (3.0 - b * X[i] * X[i]) 
 *	where b is the input value to be inverted
 *
 */
static __inline vector float _rsqrtf4(vector float value)
{
  vector float result;
  vector float x0, x0_square, x0_half;
  vector float vzero = ((vector float) { 0.0f, 0.0f, 0.0f, 0.0f });

  x0 = vec_rsqrte(value);
  x0_square = vec_madd(x0, x0, vzero);
  x0_half = vec_madd(x0, ((vector float) { 0.5f, 0.5f, 0.5f, 0.5f }), vzero);
  result = vec_nmsub(value, x0_square, ((vector float) { 3.0f, 3.0f, 3.0f, 3.0f }));
  return (vec_madd(result, x0_half, vzero));
}
#endif /* _RSQRTF4_H_ */
